defmodule ReadDoc.Mixfile do
  use Mix.Project

  @description """
  A simple tool to insert docstrings into files (typically markdown files like README.md)
  """
  def project do
    [
      app: :read_doc,
      version: "0.1.0",
      elixir: "~> 1.5",
      description: @description,
      start_permanent: Mix.env == :prod,
      elixirc_paths: elixirc_paths(Mix.env),
      package: package(),
      test_coverage: [tool: ExCoveralls],
      preferred_cli_env: ["coveralls": :test, "coveralls.html": :test],
      aliases: [read_doc: &read_doc/1],
      deps: deps()
    ]
  end

  # Run "mix help compile.app" to learn about applications.
  def application do
    [
      extra_applications: [:logger]
    ]
  end

  # Run "mix help deps" to learn about dependencies.
  defp deps do
    [
      # {:dep_from_hexpm, "~> 0.3.0"},
      # {:dep_from_git, git: "https://github.com/elixir-lang/my_dep.git", tag: "0.1.0"},
      {:ex_doc, "~> 0.18"},
      {:excoveralls, "~> 0.8.0", only: :test},
      {:dialyxir, "~> 0.5.1", only: :dev},
    ]
  end

  defp elixirc_paths(:test), do: ["lib", "test_support"]
  defp elixirc_paths(_),     do: ["lib"]

  defp package do
    [
      files:       [ "lib", "mix.exs", "README.md", "LICENSE", "tasks" ],
      maintainers: [
                     "Robert Dober <robert.dober@gmail.com>"
                   ],
      licenses:    [ "Apache 2 (see the file LICENSE)" ],
      links:       %{
                       "GitHub" => "https://github.com/RobertDober/read_doc",
                   }
    ]
  end

  defp read_doc(args) do
    Code.load_file "tasks/read_doc.exs"
    Mix.Tasks.ReadDoc.run(args)
  end
end
